@@ -20,7 +20,7 @@ squarem <- function(par, fixptfn, objfn, ... , control=list()) {
 # method = 1, 2, or 3, indicating the type of steplength to be used
 #
 #
-control.default <- list(K = 1, method=3, square=TRUE, step.min0=1, step.max0=1, mstep=4, kr=1, objfn.inc=1,
+control.default <- list(K = 1, method=3, minimize=TRUE, square=TRUE, step.min0=1, step.max0=1, mstep=4, kr=1, objfn.inc=1,
 	tol=1.e-07, maxiter=1500, trace=FALSE, intermed=FALSE)
 namc <- names(control)
 if (!all(namc %in% names(control.default))) 
@@ -61,7 +61,7 @@ squarem1 <- function(par, fixptfn, objfn, ... , control=list()) {
 # objfn = underlying objective function which is minimized at x*
 #
 #
-control.default <- list(K=1, square=TRUE, method=3, step.min0=1, step.max0=1, mstep=4, kr=1, objfn.inc=1, 
+control.default <- list(K=1, square=TRUE, minimize=TRUE, method=3, step.min0=1, step.max0=1, mstep=4, kr=1, objfn.inc=1, 
 	tol=1.e-07, maxiter=1500, trace=FALSE, intermed=FALSE)
 
 namc <- names(control)
@@ -81,6 +81,7 @@ ctrl <- modifyList(control.default, control)
 #
     method <- ctrl$method
     maxiter <- ctrl$maxiter
+    minimize <- ctrl$minimize
     tol <- ctrl$tol
     step.min <- ctrl$step.min0
     step.max0 <- ctrl$step.max0
@@ -90,7 +91,7 @@ ctrl <- modifyList(control.default, control)
     trace <- ctrl$trace
     intermed <- ctrl$intermed
 
-if (trace) cat("Squarem-1 \n")
+if (trace) cat("Squarem-1 \n ")
 
 if (missing(objfn)) stop("\n squarem2 should be used if objective function is not available \n\n")
 
@@ -109,18 +110,18 @@ while (feval < maxiter) {
 	extrap <- TRUE
 	p1 <- try(fixptfn(p, ...),silent=TRUE)
 	feval <- feval + 1
-	if (class(p1) == "try-error" | any(is.nan(unlist(p1)))) stop("Error in function evaluation")
+	if (inherits(p1, "try-error") | any(is.nan(unlist(p1)))) stop("Error in function evaluation")
 	q1 <- p1 - p
 	sr2 <- crossprod(q1)
-	if (sqrt(sr2) < tol) break
+	if (sqrt(sr2) < tol) {p <- p1; break}
 
 	p2 <- try(fixptfn(p1, ...),silent=TRUE)
 	feval <- feval + 1
-	if (class(p2) == "try-error" | any(is.nan(unlist(p2)))) stop("Error in function evaluation")
+	if (inherits(p2, "try-error") | any(is.nan(unlist(p2)))) stop("Error in function evaluation")
 
 	q2 <- p2 - p1
 	sq2 <- sqrt(crossprod(q2))
-	if (sq2 < tol) break
+	if (sq2 < tol) {p <- p2; break}
 	sv2 <- crossprod(q2-q1)
 	srv <- crossprod(q1, q2-q1)
 
@@ -133,7 +134,7 @@ while (feval < maxiter) {
 		feval <- feval + 1
 	}
 
-	if (class(p.new) == "try-error" | any(is.nan(p.new))) {
+	if (inherits(p.new, "try-error") | any(is.nan(p.new))) {
 	p.new <- p2
 	lnew <- try(objfn(p2, ...), silent=TRUE)
 	leval <- leval + 1
@@ -145,8 +146,8 @@ while (feval < maxiter) {
 			lnew <- try(objfn(p.new, ...), silent=TRUE)
 			leval <- leval + 1
 		} else lnew <- lold
-		if (class(lnew) == "try-error" | is.nan(lnew) | 
-		(lnew > lold + objfn.inc)) {
+		non.monotone <- if (minimize) lnew > lold + objfn.inc else lnew < lold - objfn.inc
+		if (inherits(lnew, "try-error") | is.nan(lnew) | non.monotone) {
 			p.new <- p2
 			lnew <- try(objfn(p2, ...), silent=TRUE)
 			leval <- leval + 1
@@ -174,7 +175,7 @@ while (feval < maxiter) {
 
 rownames(p.inter) <- NULL
 if (!intermed) return(list(par=p, value.objfn=lold, iter= iter, fpevals=feval, objfevals = leval, convergence=conv))
-   else return(list(par=p, value.objfn=lold, iter= iter, fpevals=feval, objfevals = leval, convergence=conv, p.inter=p.inter))
+   else return(list(par=p, value.objfn=lold, iter= iter, fpevals=feval, objfevals = leval, convergence=conv, p.intermed=p.inter))
 }
 ###################################################################
 squarem2 <- function(par, fixptfn, ... , control=list() ) {
@@ -186,7 +187,7 @@ squarem2 <- function(par, fixptfn, ... , control=list() ) {
 #
 # See Varadhan and Roland (Scandinavian J Statistics 2008)
 #
-control.default <- list(K=1, square=TRUE, method=3, step.min0=1, step.max0=1, mstep=4, kr=1, objfn.inc=1, 
+control.default <- list(K=1, square=TRUE, minimize=TRUE, method=3, step.min0=1, step.max0=1, mstep=4, kr=1, objfn.inc=1, 
 	tol=1.e-07, maxiter=1500, trace=FALSE, intermed=FALSE)
 
 namc <- names(control)
@@ -213,28 +214,31 @@ ctrl <- modifyList(control.default, control)
     mstep <- ctrl$mstep
     trace <- ctrl$trace
 if (trace) cat("Squarem-2 \n")
-
+    intermed <- ctrl$intermed
+    
 iter <- 1
 feval <- 0
 kount <- 0
 conv <- TRUE
+p.inter <- rep(NA, length(par)+1)
 
 while (feval < maxiter) {
 	extrap <- TRUE
 	p1 <- try(fixptfn(par, ...),silent=TRUE)
 	feval <- feval + 1
-	if (class(p1) == "try-error" | any(is.nan(unlist(p1)))) break
+	if (inherits(p1, "try-error") | any(is.nan(unlist(p1)))) break
 	q1 <- p1 - par
 	sr2 <- crossprod(q1)
-	if (sqrt(sr2) < tol) break
+	p.inter <- rbind(p.inter, c(par, sqrt(sr2)))
+	if (sqrt(sr2) < tol) {par <- p1; break}
 
 	p2 <- try(fixptfn(p1, ...),silent=TRUE)
 	feval <- feval + 1
-	if (class(p2) == "try-error" | any(is.nan(unlist(p2)))) break
+	if (inherits(p2, "try-error") | any(is.nan(unlist(p2)))) break
 	q2 <- p2 - p1
 	sq2 <- sqrt(crossprod(q2))
 	res <- sq2
-	if (sq2 < tol) break
+	if (sq2 < tol) {par <- p2; break}
 	sv2 <- crossprod(q2-q1)
 	srv <- crossprod(q1, q2-q1)	
 
@@ -246,7 +250,7 @@ while (feval < maxiter) {
 	if (abs(alpha - 1) > 0.01 ) {
 	ptmp <- try(fixptfn(p.new, ...),silent=TRUE)   # stabilization step
 	feval <- feval + 1
-		if (class(ptmp) == "try-error" | any(is.nan(unlist(ptmp))) ) {
+		if (inherits(ptmp, "try-error") | any(is.nan(unlist(ptmp))) ) {
 		p.new <- p2
 		if (alpha == step.max) step.max <- max(step.max0, step.max/mstep)
 		alpha <- 1 
@@ -272,9 +276,12 @@ while (feval < maxiter) {
 	par <- p.new
 	iter <- iter+1
 }
-	if (feval >= maxiter) conv <- FALSE
+p.inter <- p.inter[-1, ]
+if (feval >= maxiter) conv <- FALSE
 
-return(list(par=par, value.objfn=NA, iter = iter, fpevals=feval, objfevals = 0, convergence=conv))
+if (intermed) return(list(par=par, value.objfn=NA, iter = iter, fpevals=feval, objfevals = 0, 
+                          convergence=conv, p.intermed=p.inter))
+else return(list(par=par, value.objfn=NA, iter = iter, fpevals=feval, objfevals = 0, convergence=conv))
 
 }
 #######################################################################
@@ -291,7 +298,7 @@ cyclem1 <- function(par, fixptfn, objfn, control=list(), ...) {
 # for which the solution: F(x*) = x* is sought
 # objfn = underlying objective function which is minimized at x*
 # 
-control.default <- list(K=2, method="rre", square=TRUE, step.min0=1, step.max0=1, mstep=4, kr=1, objfn.inc=1, 
+control.default <- list(K=2, method="rre", minimize=TRUE, square=TRUE, step.min0=1, step.max0=1, mstep=4, kr=1, objfn.inc=1, 
 	tol=1.e-07, maxiter=1500, trace=FALSE, intermed=FALSE)
 
 namc <- names(control)
@@ -338,7 +345,7 @@ while (feval < maxiter & res > tol) {
 	p[,1] <- par
 	for (i in 2:(K+2)){
 	p[,i] <- try(fixptfn(p[,i-1], ...), silent=TRUE)
-	if (class(p[,i]) == "try-error" | any(is.nan(unlist(p[,i]))) | any(!is.numeric(p[,i]))) stop("Error in function evaluation \n")
+	if (inherits(p[,i], "try-error") | any(is.nan(unlist(p[,i]))) | any(!is.numeric(p[,i]))) stop("Error in function evaluation \n")
 	U[,i-1] <- p[, i] - p[, i-1]
 	}
 	feval <- feval + K + 1 
@@ -346,7 +353,7 @@ while (feval < maxiter & res > tol) {
 
 	if (method == "rre"){
 	coef <- try(solve(qr(crossprod(U), LAPACK=TRUE, tol=1.e-14), rep(1,K+1)), silent=TRUE)
-	if (class(coef) == "try-error" | any(is.nan(coef)) ) {
+	if (inherits(coef, "try-error") | any(is.nan(coef)) ) {
 		extrap <- FALSE
 	} else {
 		if (abs(sum(coef)) < 1.e-07) extrap <- FALSE
@@ -355,7 +362,7 @@ while (feval < maxiter & res > tol) {
 	}
 	if (method == "mpe"){
 	coef <- try(solve(qr(U[,-(K+1)], LAPACK=TRUE, tol=1.e-14), -U[,K+1]), silent=TRUE)
-	if (class(coef) == "try-error" | any(is.nan(coef))) {
+	if (inherits(coef, "try-error") | any(is.nan(coef))) {
 		extrap <- FALSE
 	} else {
 		coef <- c(coef, 1)
@@ -378,7 +385,7 @@ while (feval < maxiter & res > tol) {
 		if (K > 1) {
 		for (i in 1:(K-1)) {
 			p <- try(cbind(p, fixptfn(p[,i+K+1],...)), silent=TRUE)
-			if (class(p) == "try-error" | any(is.nan(unlist(p))) | any(!is.numeric(p)))  stop("Error in function evaluation \n")
+			if (inherits(p, "try-error")| any(is.nan(unlist(p))) | any(!is.numeric(p)))  stop("Error in function evaluation \n")
 			}
 		feval <- feval + K - 1
 		}
@@ -395,7 +402,7 @@ while (feval < maxiter & res > tol) {
 		res <- sqrt(c(crossprod(ptmp - pnew)))
 		feval <- feval + 1
 		} 
-	if (class(ptmp) == "try-error" | any(is.nan(unlist(ptmp))) ) {
+	if (inherits(ptmp, "try-error") | any(is.nan(unlist(ptmp))) ) {
 		pnew <- p[, ncol(p)]
 		res <- sqKp1
 		extrap <- FALSE
@@ -403,7 +410,9 @@ while (feval < maxiter & res > tol) {
 		
 	lnew <- try(objfn(pnew, ...), silent=TRUE)
 	leval <- leval + 1
-	if (class(lnew) == "try-error" | is.nan(lnew) | (lnew > lold + objfn.inc)) {
+	non.monotone <- if(ctrl$minimize) lnew > lold+objfn.inc else lnew < lold-objfn.inc
+	
+	if (inherits(lnew, "try-error") | is.nan(lnew) | non.monotone) {
 		pnew <- p[, ncol(p)]
 		res <- sqKp1
 		extrap <- FALSE
@@ -432,7 +441,7 @@ cyclem2 <- function(par, fixptfn, control=list(), ...) {
 # for which the solution: F(x*) = x* is sought
 # method = reduced-rank or minimal-polynomial extrapolation
 #
-control.default <- list(K=2, method="rre", square=TRUE, step.min0=1, step.max0=1, mstep=4, kr=1, objfn.inc=1, 
+control.default <- list(K=2, method="rre", minimize=TRUE, square=TRUE, step.min0=1, step.max0=1, mstep=4, kr=1, objfn.inc=1, 
 	tol=1.e-07, maxiter=1500, trace=FALSE, intermed=FALSE)
 
 namc <- names(control)
@@ -474,7 +483,7 @@ while (feval < maxiter & res > tol) {
 	p[,1] <- par
 	for (i in 2:(K+2)){
 	p[,i] <- try(fixptfn(p[,i-1], ...), silent=TRUE)
-	if (class(p[,i]) == "try-error" | any(is.nan(unlist(p[,i]))) | any(!is.numeric(p[,i]))) stop("Error in function evaluation")
+	if (inherits(p[,i], "try-error") | any(is.nan(unlist(p[,i]))) | any(!is.numeric(p[,i]))) stop("Error in function evaluation")
 	U[,i-1] <- p[, i] - p[, i-1]
 	}
 	feval <- feval + K + 1 
@@ -482,7 +491,7 @@ while (feval < maxiter & res > tol) {
 
 	if (method == "rre"){
 	coef <- try(solve(qr(crossprod(U), LAPACK=TRUE, tol=1.e-14), rep(1,K+1)), silent=TRUE)
-	if (class(coef) == "try-error" | any(is.nan(coef))) {
+	if (inherits(coef, "try-error") | any(is.nan(coef))) {
 		extrap <- FALSE
 	} else {
 		if (abs(sum(coef)) < 1.e-07) extrap <- FALSE
@@ -491,7 +500,7 @@ while (feval < maxiter & res > tol) {
 	}
 	if (method == "mpe"){
 	coef <- try(solve(qr(U[,-(K+1)], LAPACK=TRUE, tol=1.e-14), -U[,K+1]), silent=TRUE)
-	if (class(coef) == "try-error" | any(is.nan(coef))) {
+	if (inherits(coef, "try-error") | any(is.nan(coef))) {
 		extrap <- FALSE
 	} else {
 		coef <- c(coef, 1)
@@ -514,7 +523,7 @@ while (feval < maxiter & res > tol) {
 		if (K > 1) {
 		for (i in 1:(K-1)) {
 			p <- try(cbind(p, fixptfn(p[,i+K+1],...)), silent=TRUE)
-			if (class(p) == "try-error" | any(is.nan(unlist(p))) | any(!is.numeric(p))) stop("Error in function evaluation")
+			if (inherits(p, "try-error") | any(is.nan(unlist(p))) | any(!is.numeric(p))) stop("Error in function evaluation")
 			}
 		feval <- feval + K - 1
 		}
@@ -529,7 +538,7 @@ while (feval < maxiter & res > tol) {
 		ptemp <- try(fixptfn(pnew, ...), silent=TRUE)
 		
       feval <- feval + 1
-	if(class(ptemp)=="try-error" | any(is.nan(ptemp)) ){
+	if(inherits(ptemp, "try-error") | any(is.nan(ptemp)) ){
 		par <- p[, ncol(p)]
 		res <- sqKp1
 		extrap <- FALSE
@@ -561,7 +570,7 @@ return(list(par=par, value.objfn=NA, iter = iter, fpevals=feval, objfevals = 0,
 #
 fpiter <- function(par, fixptfn, objfn=NULL, control=list( ), ...){
 
-control.default <- list(tol=1.e-07, maxiter=5000, trace=FALSE)
+control.default <- list(tol=1.e-07, maxiter=5000, trace=FALSE, intermed=FALSE)
 namc <- names(control)
 if (!all(namc %in% names(control.default))) 
     stop("unknown names in control: ", namc[!(namc %in% names(control.default))])
@@ -575,7 +584,8 @@ ctrl <- modifyList(control.default, control)
     tol <- ctrl$tol
     maxiter <- ctrl$maxiter
     trace <- ctrl$trace
-
+    intermed <- ctrl$intermed
+    
 if (trace) cat("fpiter \n")
 
 iter <- 1
@@ -583,10 +593,13 @@ resid <- rep(NA,1)
 objeval <- 0
 conv <- FALSE
 
+if(intermed) p.inter <- rep(NA, 1+length(par))
+  
 while (iter < maxiter) {
 
 p.new <- fixptfn(par, ...)
 res <- sqrt(crossprod(p.new - par))
+if(intermed) p.inter <- rbind(p.inter, c(par, res))
 
 if ( res < tol) {conv <- TRUE; break}
 
@@ -597,11 +610,13 @@ if (trace) {
   par <- p.new
 iter <- iter+1
 }
-
+if(intermed) p.inter <- p.inter[-1,]
 loglik.best <-  if (!is.null(objfn)) objfn(par, ...) else NA
 
-return(list(par=par, value.objfn=loglik.best, fpevals=iter, objfevals = objeval, convergence=conv))
+if(!intermed) return(list(par=par, value.objfn=loglik.best, fpevals=iter, objfevals = objeval, 
+            convergence=conv))
+else return(list(par=par, value.objfn=loglik.best, fpevals=iter, objfevals = objeval, 
+                         p.intermed=p.inter, convergence=conv))
 }
 
 ###################################################################
-
